@@ -1,4 +1,4 @@
-/* Copyright (C) 2006-2016 J.F.Dockes
+/* Copyright (C) 2006-2024 J.F.Dockes
  *
  * This library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Lesser General Public
@@ -15,11 +15,11 @@
  * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA
  *   02110-1301 USA
  */
-#include "libupnpp/config.h"
+#include "config.h"
 
 #include "libupnpp/control/avtransport.hxx"
 
-#include <stdlib.h>
+#include <cstdlib>
 #include <upnp.h>
 
 #include <ostream>
@@ -116,8 +116,7 @@ static AVTransport::PlayMode stringToPlayMode(const string& s)
     }
 }
 
-void AVTransport::evtCallback(
-    const std::unordered_map<std::string, std::string>& props)
+void AVTransport::evtCallback(const std::unordered_map<std::string, std::string>& props)
 {
     LOGDEB1("AVTransport::evtCallback:" << "\n");
     auto reporter = getReporter();
@@ -126,59 +125,50 @@ void AVTransport::evtCallback(
         return;
     }
 
-    for (const auto& prop : props) {
-        if (prop.first.compare("LastChange")) {
+    for (const auto& [propname, propvalue] : props) {
+        if (propname != "LastChange") {
             LOGINF("AVTransport:event: var not lastchange: "
-                   << prop.first << " -> " << prop.second << "\n");
+                   << propname << " -> " << propvalue << "\n");
             continue;
         }
-        LOGDEB1("AVTransport:event: " << prop.first << " -> " << prop.second << "\n");
+        LOGDEB1("AVTransport:event: " << propname << " -> " << propvalue << "\n");
 
         std::unordered_map<std::string, std::string> changes;
-        if (!decodeAVLastChange(prop.second, changes)) {
-            LOGERR("AVTransport::evtCallback: bad LastChange value: " << prop.second << "\n");
+        if (!decodeAVLastChange(propvalue, changes)) {
+            LOGERR("AVTransport::evtCallback: bad LastChange value: " << propvalue << "\n");
             return;
         }
-        for (const auto& varchg : changes) {
-            const std::string& varnm{varchg.first};
-            const std::string& varvalue{varchg.second};
 
-            if (!varnm.compare("TransportState")) {
+        for (const auto& [varnm, varvalue] : changes) {
+            if (varnm == "TransportState") {
                 reporter->changed(varnm.c_str(), stringToTpState(varvalue));
-
-            } else if (!varnm.compare("TransportStatus")) {
+            } else if (varnm == "TransportStatus") {
                 reporter->changed(varnm.c_str(), stringToTpStatus(varvalue));
-
-            } else if (!varnm.compare("CurrentPlayMode")) {
+            } else if (varnm == "CurrentPlayMode") {
                 reporter->changed(varnm.c_str(), stringToPlayMode(varvalue));
-
-            } else if (!varnm.compare("CurrentTransportActions")) {
+            } else if (varnm == "CurrentTransportActions") {
                 int iacts;
                 if (!CTAStringToBits(varvalue, iacts))
                     reporter->changed(varnm.c_str(), iacts);
-
-            } else if (!varnm.compare("CurrentTrackURI") ||
-                       !varnm.compare("AVTransportURI") ||
-                       !varnm.compare("NextAVTransportURI")) {
+            } else if (varnm == "CurrentTrackURI" ||
+                       varnm == "AVTransportURI" ||
+                       varnm == "NextAVTransportURI") {
                 reporter->changed(varnm.c_str(), varvalue.c_str());
-
-            } else if (!varnm.compare("TransportPlaySpeed") ||
-                       !varnm.compare("CurrentTrack") ||
-                       !varnm.compare("NumberOfTracks") ||
-                       !varnm.compare("RelativeCounterPosition") ||
-                       !varnm.compare("AbsoluteCounterPosition") ||
-                       !varnm.compare("InstanceID")) {
+            } else if (varnm == "TransportPlaySpeed" ||
+                       varnm == "CurrentTrack" ||
+                       varnm == "NumberOfTracks" ||
+                       varnm == "RelativeCounterPosition" ||
+                       varnm == "AbsoluteCounterPosition" ||
+                       varnm == "InstanceID") {
                 reporter->changed(varnm.c_str(), atoi(varvalue.c_str()));
-
-            } else if (!varnm.compare("CurrentMediaDuration") ||
-                       !varnm.compare("CurrentTrackDuration") ||
-                       !varnm.compare("RelativeTimePosition") ||
-                       !varnm.compare("AbsoluteTimePosition")) {
+            } else if (varnm == "CurrentMediaDuration" ||
+                       varnm == "CurrentTrackDuration" ||
+                       varnm == "RelativeTimePosition" ||
+                       varnm == "AbsoluteTimePosition") {
                 reporter->changed(varnm.c_str(), upnpdurationtos(varvalue));
-
-            } else if (!varnm.compare("AVTransportURIMetaData") ||
-                       !varnm.compare("NextAVTransportURIMetaData") ||
-                       !varnm.compare("CurrentTrackMetaData")) {
+            } else if (varnm == "AVTransportURIMetaData" ||
+                       varnm == "NextAVTransportURIMetaData" ||
+                       varnm == "CurrentTrackMetaData") {
                 UPnPDirContent meta;
                 if (!varvalue.empty() && !meta.parse(varvalue)) {
                     LOGERR("AVTransport event: bad metadata: [" << varvalue << "]" << "\n");
@@ -188,15 +178,14 @@ void AVTransport::evtCallback(
                         reporter->changed(varnm.c_str(), meta.m_items[0]);
                     }
                 }
-            } else if (!varnm.compare("PlaybackStorageMedium") ||
-                       !varnm.compare("PossiblePlaybackStorageMedia") ||
-                       !varnm.compare("RecordStorageMedium") ||
-                       !varnm.compare("PossibleRecordStorageMedia") ||
-                       !varnm.compare("RecordMediumWriteStatus") ||
-                       !varnm.compare("CurrentRecordQualityMode") ||
-                       !varnm.compare("PossibleRecordQualityModes")) {
+            } else if (varnm == "PlaybackStorageMedium" ||
+                       varnm == "PossiblePlaybackStorageMedia" ||
+                       varnm == "RecordStorageMedium" ||
+                       varnm == "PossibleRecordStorageMedia" ||
+                       varnm == "RecordMediumWriteStatus" ||
+                       varnm == "CurrentRecordQualityMode" ||
+                       varnm == "PossibleRecordQualityModes") {
                 reporter->changed(varnm.c_str(), varvalue.c_str());
-
             } else {
                 LOGDEB1("AVTransport event: unknown variable: name [" <<
                         varnm << "] value [" << varvalue << "\n");
@@ -303,12 +292,17 @@ int AVTransport::getTransportInfo(TransportInfo& info, int instanceID)
     return 0;
 }
 
-int AVTransport::getPositionInfo(PositionInfo& info, int instanceID)
+int AVTransport::getPositionInfo(PositionInfo& info, int instanceID, int timeoutms)
 {
     SoapOutgoing args(getServiceType(), "GetPositionInfo");
     args("InstanceID", SoapHelp::i2s(instanceID));
     SoapIncoming data;
-    int ret = runAction(args, data);
+    ActionOptions opts;
+    if (timeoutms >= 0) {
+        opts.active_options |= AOM_TIMEOUTMS;
+        opts.timeoutms = timeoutms;
+    }
+    int ret = runAction(args, data, &opts);
     if (ret != UPNP_E_SUCCESS) {
         return ret;
     }
