@@ -1,4 +1,4 @@
-/* Copyright (C) 2006-2016 J.F.Dockes
+/* Copyright (C) 2006-2024 J.F.Dockes
  *
  * This library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Lesser General Public
@@ -17,17 +17,17 @@
  */
 // An XML parser which constructs an UPnP device object from the
 // device descriptor
-#include "libupnpp/config.h"
+#include "config.h"
 
 #include "description.hxx"
 
 #include <algorithm>
 
-#include <string.h>
+#include <cstring>
 #include <upnp.h>
 
 #include "libupnpp/upnpplib.hxx"
-#include "libupnpp/expatmm.hxx"
+#include "libupnpp/expatmm.h"
 #include "libupnpp/upnpp_p.hxx"
 #include "libupnpp/smallut.h"
 #include "libupnpp/log.hxx"
@@ -43,7 +43,8 @@ public:
         : inputRefXMLParser(input), m_device(device) {}
 
 protected:
-    virtual void EndElement(const XML_Char *name) {
+    void EndElement(const XML_Char* name) override
+    {
         trimstring(m_chardata, " \t\n\r");
 
         // If deviceList is in the current tag path, this is an embedded device.
@@ -61,7 +62,7 @@ protected:
             dev->services.push_back(m_tservice);
             m_tservice.clear();
         } else if (!strcmp(name, "device")) {
-            if (ismain == false) {
+            if (!ismain) {
                 m_device.embedded.push_back(m_tdevice);
             }
             m_tdevice.clear();
@@ -92,7 +93,8 @@ protected:
         m_chardata.clear();
     }
 
-    virtual void CharacterData(const XML_Char *s, int len) {
+    void CharacterData(const XML_Char* s, int len) override
+    {
         if (s == 0 || *s == 0)
             return;
 
@@ -116,6 +118,7 @@ UPnPDeviceDesc::UPnPDeviceDesc(const string& url, const string& description)
     UPnPDeviceParser mparser(description, *this);
     if (!mparser.Parse())
         return;
+    descURL = url;
     if (URLBase.empty()) {
         // The standard says that if the URLBase value is empty, we
         // should use the url the description was retrieved
@@ -126,6 +129,7 @@ UPnPDeviceDesc::UPnPDeviceDesc(const string& url, const string& description)
     }
     for (auto& dev: embedded) {
         dev.URLBase = URLBase;
+        dev.ok = true;
     }
     
     ok = true;
@@ -142,7 +146,8 @@ public:
     }
 
 protected:
-    virtual void StartElement(const XML_Char *name, const XML_Char **) {
+    void StartElement(const XML_Char* name, const XML_Char**) override
+    {
         //LOGDEB("startElement: name [" << name << "]" << " bpos " <<
         //             XML_GetCurrentByteIndex(expat_parser) << endl);
         StackEl& lastelt = m_path.back();
@@ -169,7 +174,7 @@ protected:
         }
     }
 
-    virtual void EndElement(const XML_Char *name)
+    void EndElement(const XML_Char* name) override
     {
         string parentname;
         if (m_path.size() == 1) {
@@ -192,11 +197,7 @@ protected:
             break;
         case 'd':
             if (!strcmp(name, "direction")) {
-                if (!lastelt.data.compare("in")) {
-                    m_targ.todevice = true;
-                } else {
-                    m_targ.todevice = false;
-                }
+                m_targ.todevice = lastelt.data == "in";
             } else if (!strcmp(name, "dataType")) {
                 m_tvar.dataType = lastelt.data;
                 trimstring(m_tvar.dataType);
@@ -213,13 +214,13 @@ protected:
             break;
         case 'n':
             if (!strcmp(name, "name")) {
-                if (!parentname.compare("argument")) {
+                if (parentname == "argument") {
                     m_targ.name = lastelt.data;
                     trimstring(m_targ.name);
-                } else if (!parentname.compare("action")) {
+                } else if (parentname == "action") {
                     m_tact.name = lastelt.data;
                     trimstring(m_tact.name);
-                } else if (!parentname.compare("stateVariable")) {
+                } else if (parentname == "stateVariable") {
                     m_tvar.name = lastelt.data;
                     trimstring(m_tvar.name);
                 }
@@ -242,7 +243,7 @@ protected:
         }
     }
 
-    virtual void CharacterData(const XML_Char *s, int len)
+    void CharacterData(const XML_Char* s, int len) override
     {
         if (s == 0 || *s == 0)
             return;
@@ -256,16 +257,15 @@ private:
     UPnPServiceDesc::StateVariable m_tvar;
 };
 
-bool UPnPServiceDesc::fetchAndParseDesc(const string& urlbase,
-                                        Parsed& parsed, string *xmltxt) const
+bool UPnPServiceDesc::fetchAndParseDesc(const string& urlbase, Parsed& parsed, string *xmltxt) const
 {
     char *buf = 0;
     char contentType[LINE_SIZE];
     string url = caturl(urlbase, SCPDURL);
     int code = UpnpDownloadUrlItem(url.c_str(), &buf, contentType);
     if (code != UPNP_E_SUCCESS) {
-        LOGERR("UPnPServiceDesc::fetchAndParseDesc: error fetching " <<
-               url << " : " << LibUPnP::errAsString("", code) << endl);
+        LOGERR("UPnPServiceDesc::fetchAndParseDesc: error fetching " << url << " : " <<
+               LibUPnP::errAsString("", code) << '\n');
         return false;
     }
     if (xmltxt) {
