@@ -1,4 +1,4 @@
-/* Copyright (C) 2006-2016 J.F.Dockes
+/* Copyright (C) 2006-2024 J.F.Dockes
  *
  * This library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Lesser General Public
@@ -15,7 +15,7 @@
  * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA
  *   02110-1301 USA
  */
-#include "libupnpp/config.h"
+#include "config.h"
 
 #include "libupnpp/control/mediarenderer.hxx"
 
@@ -82,13 +82,11 @@ static bool MDAccum(std::unordered_map<string, UPnPDeviceDesc>* out,
     return true;
 }
 
-bool MediaRenderer::getDeviceDescs(vector<UPnPDeviceDesc>& devices,
-                                   const string& friendlyName)
+bool MediaRenderer::getDeviceDescs(vector<UPnPDeviceDesc>& devices, const string& friendlyName)
 {
     std::unordered_map<string, UPnPDeviceDesc> mydevs;
 
-    UPnPDeviceDirectory::Visitor visitor = bind(
-        MDAccum, &mydevs, friendlyName, _1, _2);
+    UPnPDeviceDirectory::Visitor visitor = bind(MDAccum, &mydevs, friendlyName, _1, _2);
     UPnPDeviceDirectory::getTheDir()->traverse(visitor);
     for (const auto& dev : mydevs) {
         devices.push_back(dev.second);
@@ -100,7 +98,7 @@ MediaRenderer::MediaRenderer(const UPnPDeviceDesc& desc)
     : Device(desc)
 {
     if ((m = new Internal()) == 0) {
-        LOGERR("MediaRenderer::MediaRenderer: out of memory" << endl);
+        LOGERR("MediaRenderer::MediaRenderer: out of memory" << '\n');
         return;
     }
 }
@@ -112,7 +110,7 @@ MediaRenderer::~MediaRenderer()
 
 bool MediaRenderer::hasOpenHome()
 {
-    return ohpr() ? true : false;
+    return static_cast<bool>(ohpr());
 }
 
 
@@ -154,12 +152,12 @@ RDCH MediaRenderer::rdc()
         return rdcl;
     for (const auto& service : desc()->services) {
         if (RenderingControl::isRDCService(service.serviceType)) {
-            rdcl = RDCH(new RenderingControl(*desc(), service));
+            rdcl = std::make_shared<RenderingControl>(*desc(), service);
             break;
         }
     }
     if (!rdcl)
-        LOGDEB("MediaRenderer: RenderingControl service not found" << endl);
+        LOGDEB("MediaRenderer: RenderingControl service not found" << '\n');
     m->rdc = rdcl;
     return rdcl;
 }
@@ -171,12 +169,12 @@ AVTH MediaRenderer::avt()
         return avtl;
     for (const auto& service : desc()->services) {
         if (AVTransport::isAVTService(service.serviceType)) {
-            avtl = AVTH(new AVTransport(*desc(), service));
+            avtl = std::make_shared<AVTransport>(*desc(), service);
             break;
         }
     }
     if (!avtl)
-        LOGDEB("MediaRenderer: AVTransport service not found" << endl);
+        LOGDEB("MediaRenderer: AVTransport service not found" << '\n');
     m->avt = avtl;
     return avtl;
 }
@@ -188,13 +186,13 @@ CNMH MediaRenderer::conman()
         return cnml;
     for (const auto& servdesc : desc()->services) {
         if (ConnectionManager::isConManService(servdesc.serviceType)) {
-            cnml = CNMH(new ConnectionManager(servdesc.serviceType));
+            cnml = std::make_shared<ConnectionManager>(servdesc.serviceType);
             cnml->initFromDescription(*desc());
             break;
         }
     }
     if (!cnml)
-        LOGDEB("MediaRenderer: ConnectionManager service not found" << endl);
+        LOGDEB("MediaRenderer: ConnectionManager service not found" << '\n');
     m->cnm = cnml;
     return cnml;
 }
@@ -206,12 +204,12 @@ OHPRH MediaRenderer::ohpr()
         return ohprl;
     for (const auto& service : desc()->services) {
         if (OHProduct::isOHPrService(service.serviceType)) {
-            ohprl = OHPRH(new OHProduct(*desc(), service));
+            ohprl = std::make_shared<OHProduct>(*desc(), service);
             break;
         }
     }
     if (!ohprl)
-        LOGDEB("MediaRenderer: OHProduct service not found" << endl);
+        LOGDEB("MediaRenderer: OHProduct service not found" << '\n');
     m->ohpr = ohprl;
     return ohprl;
 }
@@ -223,12 +221,12 @@ OHPLH MediaRenderer::ohpl()
         return ohpll;
     for (const auto& service : desc()->services) {
         if (OHPlaylist::isOHPlService(service.serviceType)) {
-            ohpll = OHPLH(new OHPlaylist(*desc(), service));
+            ohpll = std::make_shared<OHPlaylist>(*desc(), service);
             break;
         }
     }
     if (!ohpll)
-        LOGDEB("MediaRenderer: OHPlaylist service not found" << endl);
+        LOGDEB("MediaRenderer: OHPlaylist service not found" << '\n');
     m->ohpl = ohpll;
     return ohpll;
 }
@@ -240,12 +238,12 @@ OHRCH MediaRenderer::ohrc()
         return ohrcl;
     for (const auto& service : desc()->services) {
         if (OHReceiver::isOHRcService(service.serviceType)) {
-            ohrcl = OHRCH(new OHReceiver(*desc(), service));
+            ohrcl = std::make_shared<OHReceiver>(*desc(), service);
             break;
         }
     }
     if (!ohrcl)
-        LOGDEB("MediaRenderer: OHReceiver service not found" << endl);
+        LOGDEB("MediaRenderer: OHReceiver service not found" << '\n');
     m->ohrc = ohrcl;
     return ohrcl;
 }
@@ -257,12 +255,12 @@ OHRDH MediaRenderer::ohrd()
         return handle;
     for (const auto& service : desc()->services) {
         if (OHRadio::isOHRdService(service.serviceType)) {
-            handle = OHRDH(new OHRadio(*desc(), service));
+            handle = std::make_shared<OHRadio>(*desc(), service);
             break;
         }
     }
     if (!handle)
-        LOGDEB("MediaRenderer: OHRadio service not found" << endl);
+        LOGDEB("MediaRenderer: OHRadio service not found" << '\n');
     m->ohrd = handle;
     return handle;
 }
@@ -274,12 +272,12 @@ OHIFH MediaRenderer::ohif()
         return handle;
     for (const auto& service : desc()->services) {
         if (OHInfo::isOHInfoService(service.serviceType)) {
-            handle = OHIFH(new OHInfo(*desc(), service));
+            handle = std::make_shared<OHInfo>(*desc(), service);
             break;
         }
     }
     if (!handle)
-        LOGDEB("MediaRenderer: OHInfo service not found" << endl);
+        LOGDEB("MediaRenderer: OHInfo service not found" << '\n');
     m->ohif = handle;
     return handle;
 }
@@ -291,12 +289,12 @@ OHSNH MediaRenderer::ohsn()
         return handle;
     for (const auto& service : desc()->services) {
         if (OHSender::isOHSenderService(service.serviceType)) {
-            handle = OHSNH(new OHSender(*desc(), service));
+            handle = std::make_shared<OHSender>(*desc(), service);
             break;
         }
     }
     if (!handle)
-        LOGDEB("MediaRenderer: OHSender service not found" << endl);
+        LOGDEB("MediaRenderer: OHSender service not found" << '\n');
     m->ohsn = handle;
     return handle;
 }
@@ -308,12 +306,12 @@ OHTMH MediaRenderer::ohtm()
         return ohtml;
     for (const auto& service : desc()->services) {
         if (OHTime::isOHTMService(service.serviceType)) {
-            ohtml = OHTMH(new OHTime(*desc(), service));
+            ohtml = std::make_shared<OHTime>(*desc(), service);
             break;
         }
     }
     if (!ohtml)
-        LOGDEB("MediaRenderer: OHTime service not found" << endl);
+        LOGDEB("MediaRenderer: OHTime service not found" << '\n');
     m->ohtm = ohtml;
     return ohtml;
 }
@@ -325,12 +323,12 @@ OHVLH MediaRenderer::ohvl()
         return ohvll;
     for (const auto& service : desc()->services) {
         if (OHVolume::isOHVLService(service.serviceType)) {
-            ohvll = OHVLH(new OHVolume(*desc(), service));
+            ohvll = std::make_shared<OHVolume>(*desc(), service);
             break;
         }
     }
     if (!ohvll)
-        LOGDEB("MediaRenderer: OHVolume service not found" << endl);
+        LOGDEB("MediaRenderer: OHVolume service not found" << '\n');
     m->ohvl = ohvll;
     return ohvll;
 }
