@@ -1,4 +1,4 @@
-/* Copyright (C) 2006-2016 J.F.Dockes
+/* Copyright (C) 2006-2024 J.F.Dockes
  *
  * This library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Lesser General Public
@@ -15,12 +15,12 @@
  * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA
  *   02110-1301 USA
  */
-#include "libupnpp/config.h"
+#include "config.h"
 
 #include "libupnpp/control/ohradio.hxx"
 
-#include <stdlib.h>
-#include <string.h>
+#include <cstdlib>
+#include <cstring>
 #include <upnp.h>
 
 #include <functional>
@@ -33,7 +33,7 @@
 #include "libupnpp/control/cdircontent.hxx"
 #include "libupnpp/control/service.hxx"
 #include "libupnpp/log.hxx"
-#include "libupnpp/expatmm.hxx"
+#include "libupnpp/expatmm.h"
 #include "libupnpp/soaphelp.hxx"
 #include "libupnpp/upnpp_p.hxx"
 
@@ -57,63 +57,60 @@ bool OHRadio::serviceTypeMatch(const std::string& tp)
     return isOHRdService(tp);
 }
 
-int OHRadio::decodeMetadata(const string& who,
-                            const string &didl, UPnPDirObject *dirent)
+int OHRadio::decodeMetadata(const string& who, const string &didl, UPnPDirObject *dirent)
 {
+    if (didl.empty()) {
+        // Actually not always an error probably, don't log it
+        return UPNP_E_BAD_RESPONSE;
+    }        
     UPnPDirContent dir;
     if (!dir.parse(didl)) {
-        LOGERR("OHRadio::decodeMetadata: " << who << ": didl parse failed: "
-               << didl << endl);
+        LOGERR("OHRadio::decodeMetadata: " << who << ": didl parse failed for: [" << didl << "]\n");
         return UPNP_E_BAD_RESPONSE;
     }
     if (dir.m_items.size() != 1) {
         LOGERR("OHRadio::decodeMetadata: " << who << ": " << dir.m_items.size()
-               << " items in response: [" << didl << "]" << endl);
+               << " items in response: [" << didl << "]" << '\n');
         return UPNP_E_BAD_RESPONSE;
     }
     *dirent = dir.m_items[0];
     return 0;
 }
 
-void OHRadio::evtCallback(
-    const std::unordered_map<std::string, std::string>& props)
+void OHRadio::evtCallback(const std::unordered_map<std::string, std::string>& props)
 {
-    LOGDEB1("OHRadio::evtCallback: getReporter(): " << getReporter() << endl);
-    for (std::unordered_map<std::string, std::string>::const_iterator it =
-                props.begin(); it != props.end(); it++) {
+    LOGDEB1("OHRadio::evtCallback: getReporter(): " << getReporter() << '\n');
+    for (const auto& [propname, propvalue] : props) {
         if (!getReporter()) {
-            LOGDEB1("OHRadio::evtCallback: " << it->first << " -> "
-                    << it->second << endl);
+            LOGDEB1("OHRadio::evtCallback: " << propname << " -> " << propvalue << '\n');
             continue;
         }
 
-        if (!it->first.compare("Id") ||
-                !it->first.compare("ChannelsMax")) {
-            getReporter()->changed(it->first.c_str(), atoi(it->second.c_str()));
-        } else if (!it->first.compare("IdArray")) {
+        if (propname == "Id" || propname == "ChannelsMax") {
+            getReporter()->changed(propname.c_str(), atoi(propvalue.c_str()));
+        } else if (propname == "IdArray") {
             // Decode IdArray. See how we call the client
             vector<int> v;
-            ohplIdArrayToVec(it->second, &v);
-            getReporter()->changed(it->first.c_str(), v);
-        } else if (!it->first.compare("ProtocolInfo") ||
-                   !it->first.compare("Uri")) {
-            getReporter()->changed(it->first.c_str(), it->second.c_str());
-        } else if (!it->first.compare("Metadata")) {
+            ohplIdArrayToVec(propvalue, &v);
+            getReporter()->changed(propname.c_str(), v);
+        } else if (propname == "ProtocolInfo" || propname == "Uri") {
+            getReporter()->changed(propname.c_str(), propvalue.c_str());
+        } else if (propname == "Metadata") {
             /* Metadata is a didl-lite string */
             UPnPDirObject dirent;
-            if (decodeMetadata("evt", it->second, &dirent) == 0) {
-                getReporter()->changed(it->first.c_str(), dirent);
+            if (decodeMetadata("evt", propvalue, &dirent) == 0) {
+                getReporter()->changed(propname.c_str(), dirent);
             } else {
                 LOGDEB("OHRadio:evtCallback: bad metadata in event\n");
             }
-        } else if (!it->first.compare("TransportState")) {
+        } else if (propname == "TransportState") {
             OHPlaylist::TPState tp;
-            OHPlaylist::stringToTpState(it->second, &tp);
-            getReporter()->changed(it->first.c_str(), int(tp));
+            OHPlaylist::stringToTpState(propvalue, &tp);
+            getReporter()->changed(propname.c_str(), int(tp));
         } else {
-            LOGERR("OHRadio event: unknown variable: name [" <<
-                   it->first << "] value [" << it->second << endl);
-            getReporter()->changed(it->first.c_str(), it->second.c_str());
+            LOGERR("OHRadio event: unknown variable: name [" << propname << "] value [" <<
+                   propvalue << '\n');
+            getReporter()->changed(propname.c_str(), propvalue.c_str());
         }
     }
 }
@@ -132,12 +129,12 @@ int OHRadio::channel(std::string* urip, UPnPDirObject *dirent)
         return ret;
     }
     if (!data.get("Uri", urip)) {
-        LOGERR("OHRadio::Read: missing Uri in response" << endl);
+        LOGERR("OHRadio::Read: missing Uri in response" << '\n');
         return UPNP_E_BAD_RESPONSE;
     }
     string didl;
     if (!data.get("Metadata", &didl)) {
-        LOGERR("OHRadio::Read: missing Uri in response" << endl);
+        LOGERR("OHRadio::Read: missing Uri in response" << '\n');
         return UPNP_E_BAD_RESPONSE;
     }
     return decodeMetadata("channel", didl, dirent);
@@ -148,9 +145,14 @@ int OHRadio::channelsMax(int *value)
     return runSimpleGet("ChannelsMax", "Value", value);
 }
 
-int OHRadio::id(int *value)
+int OHRadio::id(int *value, int timeoutms)
 {
-    return runSimpleGet("Id", "Value", value);
+    ActionOptions opts;
+    if (timeoutms >= 0) {
+        opts.active_options |= AOM_TIMEOUTMS;
+        opts.timeoutms = timeoutms;
+    }
+    return runSimpleGet("Id", "Value", value, &opts);
 }
 
 int OHRadio::idArray(vector<int> *ids, int *tokp)
@@ -162,12 +164,12 @@ int OHRadio::idArray(vector<int> *ids, int *tokp)
         return ret;
     }
     if (!data.get("Token", tokp)) {
-        LOGERR("OHRadio::idArray: missing Token in response" << endl);
+        LOGERR("OHRadio::idArray: missing Token in response" << '\n');
         return UPNP_E_BAD_RESPONSE;
     }
     string arraydata;
     if (!data.get("Array", &arraydata)) {
-        LOGINF("OHRadio::idArray: missing Array in response" << endl);
+        LOGINF("OHRadio::idArray: missing Array in response" << '\n');
         // We get this for an empty array ? This would need to be investigated
     }
     ohplIdArrayToVec(arraydata, ids);
@@ -184,7 +186,7 @@ int OHRadio::idArrayChanged(int token, bool *changed)
         return ret;
     }
     if (!data.get("Value", changed)) {
-        LOGERR("OHRadio::idArrayChanged: missing Value in response" << endl);
+        LOGERR("OHRadio::idArrayChanged: missing Value in response" << '\n');
         return UPNP_E_BAD_RESPONSE;
     }
     return 0;
@@ -209,7 +211,7 @@ int OHRadio::protocolInfo(std::string *proto)
         return ret;
     }
     if (!data.get("Value", proto)) {
-        LOGERR("OHRadio::protocolInfo: missing Value in response" << endl);
+        LOGERR("OHRadio::protocolInfo: missing Value in response" << '\n');
         return UPNP_E_BAD_RESPONSE;
     }
     return 0;
@@ -226,7 +228,7 @@ int OHRadio::read(int id, UPnPDirObject *dirent)
     }
     string didl;
     if (!data.get("Metadata", &didl)) {
-        LOGERR("OHRadio::Read: missing Metadata in response" << endl);
+        LOGERR("OHRadio::Read: missing Metadata in response" << '\n');
         return UPNP_E_BAD_RESPONSE;
     }
     return decodeMetadata("read", didl, dirent);
@@ -244,25 +246,23 @@ int OHRadio::read(int id, UPnPDirObject *dirent)
 
 class OHTrackListParser : public inputRefXMLParser {
 public:
-    OHTrackListParser(const string& input,
-                      vector<OHPlaylist::TrackListEntry>* vp)
+    OHTrackListParser(const string& input, vector<OHPlaylist::TrackListEntry>* vp)
         : inputRefXMLParser(input), m_v(vp)
     {
-        //LOGDEB("OHTrackListParser: input: " << input << endl);
+        //LOGDEB("OHTrackListParser: input: " << input << '\n');
     }
 
 protected:
-    virtual void EndElement(const XML_Char *name) {
+    void EndElement(const XML_Char* name) override
+    {
         if (!strcmp(name, "Entry")) {
             UPnPDirContent dir;
             if (!dir.parse(m_tdidl)) {
-                LOGERR("OHRadio::ReadList: didl parse failed: "
-                       << m_tdidl << endl);
+                LOGERR("OHRadio::ReadList: didl parse failed: " << m_tdidl << '\n');
                 return;
             }
             if (dir.m_items.size() != 1) {
-                LOGERR("OHRadio::ReadList: " << dir.m_items.size()
-                       << " in response!" << endl);
+                LOGERR("OHRadio::ReadList: " << dir.m_items.size() << " in response!" << '\n');
                 return;
             }
             m_tt.dirent = dir.m_items[0];
@@ -271,15 +271,16 @@ protected:
             m_tdidl.clear();
         }
     }
-    virtual void CharacterData(const XML_Char *s, int len) {
+    void CharacterData(const XML_Char* s, int len) override
+    {
         if (s == 0 || *s == 0)
             return;
         string str(s, len);
-        if (!m_path.back().name.compare("Id"))
+        if (m_path.back().name == "Id")
             m_tt.id = atoi(str.c_str());
-        else if (!m_path.back().name.compare("Uri"))
+        else if (m_path.back().name == "Uri")
             m_tt.url = str;
-        else if (!m_path.back().name.compare("Metadata"))
+        else if (m_path.back().name == "Metadata")
             m_tdidl += str;
     }
 
@@ -289,12 +290,11 @@ private:
     string m_tdidl;
 };
 
-int OHRadio::readList(const std::vector<int>& ids,
-                      vector<OHPlaylist::TrackListEntry>* entsp)
+int OHRadio::readList(const std::vector<int>& ids, vector<OHPlaylist::TrackListEntry>* entsp)
 {
     string idsparam;
-    for (vector<int>::const_iterator it = ids.begin(); it != ids.end(); it++) {
-        idsparam += SoapHelp::i2s(*it) + " ";
+    for (int id : ids) {
+        idsparam += SoapHelp::i2s(id) + " ";
     }
     entsp->clear();
 
@@ -307,7 +307,7 @@ int OHRadio::readList(const std::vector<int>& ids,
     }
     string xml;
     if (!data.get("ChannelList", &xml)) {
-        LOGERR("OHRadio::readlist: missing TrackList in response" << endl);
+        LOGERR("OHRadio::readlist: missing TrackList in response" << '\n');
         return UPNP_E_BAD_RESPONSE;
     }
     OHTrackListParser mparser(xml, entsp);
@@ -328,7 +328,7 @@ int OHRadio::setChannel(const std::string& uri, const std::string& didl)
 int OHRadio::setId(int id, const std::string& uri)
 {
     SoapOutgoing args(getServiceType(), "SetId");
-    args("Value", SoapHelp::val2s(id))
+    args("Value", std::to_string(id))
     ("Uri", uri);
     SoapIncoming data;
     return runAction(args, data);
