@@ -1,4 +1,4 @@
-/* Copyright (C) 2006-2016 J.F.Dockes
+/* Copyright (C) 2006-2024 J.F.Dockes
  *
  * This library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Lesser General Public
@@ -16,12 +16,12 @@
  *   02110-1301 USA
  */
 
-#include "libupnpp/config.h"
+#include "config.h"
 
 #include "libupnpp/control/ohreceiver.hxx"
 
-#include <stdlib.h>
-#include <string.h>
+#include <cstdlib>
+#include <cstring>
 #include <upnp.h>
 
 #include <functional>
@@ -57,35 +57,29 @@ bool OHReceiver::serviceTypeMatch(const std::string& tp)
     return isOHRcService(tp);
 }
 
-void OHReceiver::evtCallback(
-    const std::unordered_map<std::string, std::string>& props)
+void OHReceiver::evtCallback(const std::unordered_map<std::string, std::string>& props)
 {
-    LOGDEB1("OHReceiver::evtCallback:getReporter(): " << getReporter() << endl);
-    for (std::unordered_map<std::string, std::string>::const_iterator it =
-                props.begin(); it != props.end(); it++) {
+    LOGDEB1("OHReceiver::evtCallback:getReporter(): " << getReporter() << '\n');
+    for (const auto& [propname, propvalue] : props) {
         if (!getReporter()) {
-            LOGDEB1("OHReceiver::evtCallback: " << it->first << " -> "
-                    << it->second << endl);
+            LOGDEB1("OHReceiver::evtCallback: " << propname << " -> " << propvalue << '\n');
             continue;
         }
 
-        if (!it->first.compare("TransportState")) {
+        if (propname == "TransportState") {
             OHPlaylist::TPState tp;
-            OHPlaylist::stringToTpState(it->second, &tp);
-            getReporter()->changed(it->first.c_str(), int(tp));
-        } else if (!it->first.compare("Metadata")) {
-            getReporter()->changed(it->first.c_str(),
-                                   it->second.c_str());
-        } else if (!it->first.compare("Uri")) {
-            getReporter()->changed(it->first.c_str(),
-                                   it->second.c_str());
-        } else if (!it->first.compare("ProtocolInfo")) {
-            getReporter()->changed(it->first.c_str(),
-                                   it->second.c_str());
+            OHPlaylist::stringToTpState(propvalue, &tp);
+            getReporter()->changed(propname.c_str(), int(tp));
+        } else if (propname == "Metadata") {
+            getReporter()->changed(propname.c_str(), propvalue.c_str());
+        } else if (propname == "Uri") {
+            getReporter()->changed(propname.c_str(), propvalue.c_str());
+        } else if (propname == "ProtocolInfo") {
+            getReporter()->changed(propname.c_str(), propvalue.c_str());
         } else {
-            LOGERR("OHReceiver event: unknown variable: name [" <<
-                   it->first << "] value [" << it->second << endl);
-            getReporter()->changed(it->first.c_str(), it->second.c_str());
+            LOGERR("OHReceiver event: unknown variable: name [" << propname << "] value [" <<
+                   propvalue << '\n');
+            getReporter()->changed(propname.c_str(), propvalue.c_str());
         }
     }
 }
@@ -122,11 +116,11 @@ int OHReceiver::sender(string& uri, string& meta)
         return ret;
     }
     if (!data.get("Uri", &uri)) {
-        LOGERR("OHReceiver::Sender: missing Uri in response" << endl);
+        LOGERR("OHReceiver::Sender: missing Uri in response" << '\n');
         return UPNP_E_BAD_RESPONSE;
     }
     if (!data.get("Metadata", &meta)) {
-        LOGERR("OHReceiver::Sender: missing Metadata in response" << endl);
+        LOGERR("OHReceiver::Sender: missing Metadata in response" << '\n');
         return UPNP_E_BAD_RESPONSE;
     }
     return 0;
@@ -153,7 +147,7 @@ int OHReceiver::protocolInfo(std::string *proto)
         return ret;
     }
     if (!data.get("Value", proto)) {
-        LOGERR("OHReceiver::protocolInfo: missing Value in response" << endl);
+        LOGERR("OHReceiver::protocolInfo: missing Value in response" << '\n');
         return UPNP_E_BAD_RESPONSE;
     }
     return 0;
