@@ -1,4 +1,4 @@
-/* Copyright (C) 2006-2016 J.F.Dockes
+/* Copyright (C) 2006-2024 J.F.Dockes
  *
  * This library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Lesser General Public
@@ -15,14 +15,14 @@
  * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA
  *   02110-1301 USA
  */
-#define  LIBUPNPP_NEED_PACKAGE_VERSION
-#include "libupnpp/config.h"
+#include "config.h"
 
-#include <ctype.h>
-#include <stdio.h>
-#include <string.h>
-#include <time.h>
+#include <cctype>
+#include <cstdio>
+#include <cstring>
+#include <ctime>
 #include <cstdarg>
+#include <algorithm>
 
 #ifdef __MACH__
 #include <mach/clock.h>
@@ -36,6 +36,7 @@
 #include <upnpdebug.h>
 #include <netif.h>
 
+#include <algorithm>
 #include <iostream>
 #include <map>
 #include <set>
@@ -63,8 +64,19 @@ struct UPnPOptions {
     std::string ipv4;
     int port;
     int substimeout{1800};
+    int subsopstimeoutms{-1};
     std::string clientproduct;
     std::string clientversion;
+    std::string resanitizedchars{R"raw(!$'()+,)raw"};
+    int bootid{-1};
+    std::string dump() {
+        std::ostringstream out;
+        out<<"flags "<<flags<<" ifnames ["<<ifnames<<"] ipv4 ["<<ipv4<<"] port "<<port<<
+            " substimeout "<<substimeout<<" subsopstimeoutms "<<subsopstimeoutms<<
+            " clientproduct ["<<clientproduct<<"] clientversion ["<<clientversion<<
+            "] resanitizedchars ["<<resanitizedchars<<"] bootid "<< bootid;
+        return out.str();
+    }
 };
 static UPnPOptions options;
 
@@ -97,12 +109,26 @@ bool LibUPnP::init(unsigned int flags, ...)
         case UPNPPINIT_OPTION_SUBSCRIPTION_TIMEOUT:
             options.substimeout = va_arg(ap, int);
             break;
+        case UPNPPINIT_OPTION_BOOTID:
+            options.bootid = va_arg(ap, int);
+            break;
+        case UPNPPINIT_OPTION_SUBSOPS_TIMEOUTMS:
+            options.subsopstimeoutms = va_arg(ap, int);
+            break;
         case UPNPPINIT_OPTION_CLIENT_PRODUCT:
             options.clientproduct = *((std::string*)(va_arg(ap, std::string*)));
             break;
         case UPNPPINIT_OPTION_CLIENT_VERSION:
             options.clientversion = *((std::string*)(va_arg(ap, std::string*)));
             break;
+        case UPNPPINIT_OPTION_RESANITIZED_CHARS:
+        {
+            auto val = *((std::string*)(va_arg(ap, std::string*)));
+            if (!val.empty()) {
+                options.resanitizedchars = val;
+            }
+            break;
+        }
         default:
             std::cerr << "LibUPnP::init: unknown option value " << option <<"\n";
         }
@@ -115,9 +141,8 @@ bool LibUPnP::init(unsigned int flags, ...)
     return nullptr != theLib;
 }
 
-LibUPnP *LibUPnP::getLibUPnP(bool serveronly, string* hwaddr,
-                             const string ifname, const string ip,
-                             unsigned short port)
+LibUPnP *LibUPnP::getLibUPnP(
+    bool serveronly, string* hwaddr, const string ifname, const string ip, unsigned short port)
 {
     if (nullptr == theLib) {
         init(serveronly?UPNPPINIT_FLAG_SERVERONLY:0,
@@ -141,7 +166,7 @@ static int o_callback(Upnp_EventType et, CBCONST void* evp, void* cookie)
     LibUPnP *ulib = (LibUPnP *)cookie;
     if (ulib == 0) {
         // Because the asyncsearch calls uses a null cookie.
-        cerr << "o_callback: NULL ulib!" << endl;
+        cerr << "o_callback: NULL ulib!" << '\n';
         ulib = theLib;
     }
     LOGDEB1("LibUPnP::o_callback: event type: " << evTypeAsString(et) << endl);
@@ -158,7 +183,7 @@ string LibUPnP::versionString()
 {
     // We would like to print libupnp version too, but there is no way to
     // get this at runtime, and printing the compile time version is wrong.
-    return string("libupnpp ") + LIBUPNPP_PACKAGE_VERSION;
+    return string("libupnpp ") + LIBUPNPP_VERSION;
 }
 
 UpnpClient_Handle LibUPnP::Internal::getclh()
@@ -202,15 +227,18 @@ int LibUPnP::Internal::getSubsTimeout()
     return options.substimeout;
 }
 
+bool LibUPnP::Internal::reSanitizeURLs()
+{
+    return options.flags & UPNPPINIT_FLAG_RESANITIZE_URLS;
+}
+
 LibUPnP::LibUPnP()
 {
     bool serveronly = 0 != (options.flags&UPNPPINIT_FLAG_SERVERONLY);
-    LOGDEB("LibUPnP: serveronly " << serveronly << " ifnames [" <<
-           options.ifnames << "] inip [" << options.ipv4 << "] port " <<
-           options.port << endl);
+    LOGDEB("LibUPnP: serveronly " << serveronly << options.dump() << "\n");
 
     if ((m = new Internal()) == 0) {
-        LOGERR("LibUPnP::LibUPnP: out of memory" << endl);
+        LOGERR("LibUPnP::LibUPnP: out of memory" << '\n');
         return;
     }
 
@@ -232,21 +260,21 @@ LibUPnP::LibUPnP()
         m->init_error = UpnpInitWithOptions(
             options.ifnames.c_str(), options.port, flags,
             UPNP_OPTION_NETWORK_WAIT, serveronly ? 60 : 1,
+            UPNP_OPTION_BOOTID, options.bootid,
             UPNP_OPTION_END);
     }
     if (m->init_error != UPNP_E_SUCCESS) {
-        LOGERR(errAsString("UpnpInit", m->init_error) << endl);
+        LOGERR(errAsString("UpnpInit", m->init_error) << '\n');
         return;
     }
     setMaxContentLength(2000*1024);
 
 #ifdef UPNP_ENABLE_IPV6
-    LOGINF("LibUPnP: Using IPV4 " << UpnpGetServerIpAddress() << " port " <<
-           UpnpGetServerPort() << " IPV6 " << UpnpGetServerIp6Address() <<
-           " port " << UpnpGetServerPort6() << endl);
+    LOGINF("LibUPnP: Using IPV4 " << UpnpGetServerIpAddress() << " port " << UpnpGetServerPort() <<
+           " IPV6 " << UpnpGetServerIp6Address() << " port " << UpnpGetServerPort6() << '\n');
 #else
     LOGINF("LibUPnP: Using IPV4 " << UpnpGetServerIpAddress() << " port " <<
-           UpnpGetServerPort() << endl);
+           UpnpGetServerPort() << '\n');
 #endif
 
     // Client initialization is simple, just do it. Defer device
@@ -262,14 +290,24 @@ LibUPnP::LibUPnP()
                 UpnpClientSetProduct(m->clh, options.clientproduct.c_str(),
                                      options.clientversion.c_str());
             }
+#endif
+#if NPUPNP_VERSION >= 50003
+            if (options.subsopstimeoutms > 0) {
+                UpnpSubsOpsTimeoutMs(m->clh, options.subsopstimeoutms);
+            }
 #endif
             m->ok = true;
         } else {
-            LOGERR(errAsString("UpnpRegisterClient", m->init_error) << endl);
+            LOGERR(errAsString("UpnpRegisterClient", m->init_error) << '\n');
         }
     }
 }
 
+bool LibUPnP::setWebServerDocumentRoot(const std::string& rootpath)
+{
+    return UpnpSetWebServerRootDir(rootpath.c_str()) == UPNP_E_SUCCESS;
+}
+
 string LibUPnP::host()
 {
     const char *cp = UpnpGetServerIpAddress();
@@ -278,8 +316,13 @@ string LibUPnP::host()
     return string();
 }
 
-int LibUPnP::Internal::setupWebServer(const string& description,
-                                      UpnpDevice_Handle *dvh)
+string LibUPnP::port()
+{
+    unsigned short prt = UpnpGetServerPort();
+    return ulltodecstr(prt);
+}
+
+int LibUPnP::Internal::setupWebServer(const string& description, UpnpDevice_Handle *dvh)
 {
     // If we send description as a string (UPNPREG_BUF_DESC), libupnp
     // wants config_baseURL to be set, and it will serve the edited
@@ -299,9 +342,8 @@ int LibUPnP::Internal::setupWebServer(const string& description,
         o_callback, (void *)theLib, dvh);
 
     if (res != UPNP_E_SUCCESS) {
-        LOGERR("LibUPnP::setupWebServer(): " <<
-               errAsString("UpnpRegisterRootDevice2", res) << " description " <<
-               description << endl);
+        LOGERR("LibUPnP::setupWebServer(): " << errAsString("UpnpRegisterRootDevice2", res) <<
+               " description " << description << '\n');
     }
     return res;
 }
@@ -313,32 +355,23 @@ void LibUPnP::setMaxContentLength(int bytes)
 
 bool LibUPnP::setLogFileName(const std::string& fn, LogLevel level)
 {
-#if defined(HAVE_UPNPSETLOGLEVEL)
     setLogLevel(level);
     UpnpSetLogFileNames(fn.c_str(), "");
     int code = UpnpInitLog();
     if (code != UPNP_E_SUCCESS) {
-        LOGERR(errAsString("UpnpInitLog", code) << endl);
+        LOGERR(errAsString("UpnpInitLog", code) << '\n');
         return false;
     }
     return true;
-#else
-    return false;
-#endif
 }
 
 bool LibUPnP::setLogLevel(LogLevel level)
 {
-#if defined(HAVE_UPNPSETLOGLEVEL)
     UpnpSetLogLevel(Upnp_LogLevel(level));
     return true;
-#else
-    return false;
-#endif
 }
 
-void LibUPnP::Internal::registerHandler(Upnp_EventType et, Upnp_FunPtr handler,
-                                        void *cookie)
+void LibUPnP::Internal::registerHandler(Upnp_EventType et, Upnp_FunPtr handler, void *cookie)
 {
     std::unique_lock<std::mutex> lock(mutex);
     if (handler == 0) {
@@ -360,12 +393,11 @@ LibUPnP::~LibUPnP()
 {
     int error = UpnpFinish();
     if (error != UPNP_E_SUCCESS) {
-        LOGINF("LibUPnP::~LibUPnP: " << errAsString("UpnpFinish", error)
-               << endl);
+        LOGINF("LibUPnP::~LibUPnP: " << errAsString("UpnpFinish", error) << '\n');
     }
     delete m;
     m = 0;
-    LOGDEB1("LibUPnP: done" << endl);
+    LOGDEB1("LibUPnP: done" << '\n');
 }
 
 string LibUPnP::makeDevUUID(const std::string& name, const std::string& hw)
@@ -379,11 +411,8 @@ string LibUPnP::makeDevUUID(const std::string& name, const std::string& hw)
     // Make very sure that there are no colons in the hw because this
     // trips some CPs (kazoo as usual...)
     std::string nhw;
-    for (unsigned int i = 0;i < hw.size();i++) {
-        if (hw[i] != ':')
-            nhw += hw[i];
-    }
-    
+    std::remove_copy(hw.begin(), hw.end(), std::back_inserter(nhw), ':');
+
     char uuid[100];
     sprintf(uuid, "%02x%02x%02x%02x-%02x%02x-%02x%02x-%02x%02x-%s",
             digest[0]&0xff, digest[1]&0xff, digest[2]&0xff, digest[3]&0xff,
@@ -440,11 +469,11 @@ string caturl(const string& base, const string& rel)
         return rel;
     
     string out(base);
-    if (out[out.size()-1] == '/') {
-        if (rel[0] == '/')
-            out.erase(out.size()-1);
+    if (out.back() == '/') {
+        if (rel.front() == '/')
+            out.pop_back();
     } else {
-        if (rel[0] != '/')
+        if (rel.front() != '/')
             out.push_back('/');
     }
     out += rel;
@@ -457,7 +486,7 @@ string baseurl(const string& url)
     if (pos == string::npos)
         return url;
 
-    pos = url.find_first_of("/", pos + 3);
+    pos = url.find_first_of('/', pos + 3);
     if (pos == string::npos) {
         return url;
     } else {
@@ -466,8 +495,8 @@ string baseurl(const string& url)
 }
 
 static void path_catslash(string &s) {
-    if (s.empty() || s[s.length() - 1] != '/')
-        s += '/';
+    if (s.empty() || s.back() != '/')
+        s.push_back('/');
 }
 string path_getfather(const string &s)
 {
@@ -477,11 +506,11 @@ string path_getfather(const string &s)
     if (father.empty())
         return "./";
 
-    if (father[father.length() - 1] == '/') {
+    if (father.back() == '/') {
         // Input ends with /. Strip it, handle special case for root
         if (father.length() == 1)
             return father;
-        father.erase(father.length()-1);
+        father.pop_back();
     }
 
     string::size_type slp = father.rfind('/');
@@ -512,8 +541,8 @@ template <class T> bool csvToStrings(const string &s, T &tokens)
     tokens.clear();
     enum states {TOKEN, ESCAPE};
     states state = TOKEN;
-    for (unsigned int i = 0; i < s.length(); i++) {
-        switch (s[i]) {
+    for (char i : s) {
+        switch (i) {
         case ',':
             switch(state) {
             case TOKEN:
@@ -521,7 +550,7 @@ template <class T> bool csvToStrings(const string &s, T &tokens)
                 current.clear();
                 continue;
             case ESCAPE:
-                current += ',';
+                current.push_back(',');
                 state = TOKEN;
                 continue;
             }
@@ -532,7 +561,7 @@ template <class T> bool csvToStrings(const string &s, T &tokens)
                 state=ESCAPE;
                 continue;
             case ESCAPE:
-                current += '\\';
+                current.push_back('\\');
                 state = TOKEN;
                 continue;
             }
@@ -546,7 +575,7 @@ template <class T> bool csvToStrings(const string &s, T &tokens)
             case TOKEN:
                 break;
             }
-            current += s[i];
+            current.push_back(i);
         }
     }
     switch(state) {
@@ -564,6 +593,57 @@ template bool csvToStrings<vector<string> >(const string &, vector<string> &);
 template bool csvToStrings<set<string> >(const string &, set<string> &);
 
 
+// Perform additional escaping on supposedly encoded URL.
+//
+// The UPnP standards specify that URLs should encoded according to RFC 2396
+//
+// Unfortunately, the set of characters that should or should not be encoded inside a path is very
+// unclear and changed between the three URL RFCS: 1738, 2396, and the latest 3986.
+//
+// For example the "mark" characters are "unreserved" and "should not" be encoded according to
+// rfc2396:  mark = "-" | "_" | "." | "!" | "~" | "*" | "'" | "(" | ")"
+//
+// So, among others, the single quote is "unreserved" in 1738 and 2396 (no quoting), but "reserved"
+// in 3986 (should be quoted when found in data). All Python urrlib quote() versions escape it.
+//
+// In practise, unescaped single quotes do cause trouble with some renderers.
+//
+// Because they are most likely to cause confusion, this function checks/encodes the characters
+// which were in the safe list in rfc 1738, but became reserved in rfc 3986. These are all part of
+// the 3986 sub_delims list. 
+// As an aside, wikipedia explicitely states:
+//  https://en.wikipedia.org/wiki/URL_encoding: "Other characters in a URI must be percent-encoded."
+// that any character that is neither reserved (and used as such) or unreserved must be
+// percent-encoded, but I can't find this specified in the rfc, maybe it's somewhere...
+//
+// Hopefully, none of the gen_delims ones will be found unencoded because they were all reserved or
+// unsafe in 1738. 
+//
+// None of the characters that we change are likely to be used as actual syntactic delimiters in
+// UPnP URLs (hopefully).
+//
+// This function is used optionaly and not by default because encoding the * is causing trouble with
+// some renderers (Yamaha), which then emit the URI change events with a decoded *, defeating string
+// comparisons...
+// 
+// This code should probably be moved to into Control Points to be used on a per-renderer basis.
+std::string reSanitizeURL(const std::string& in)
+{
+    std::string out;
+    for (unsigned char c : in) {
+        const char *h = "0123456789ABCDEF";
+        if (c <= 0x20 || c >= 0x7f || options.resanitizedchars.find(c) != std::string::npos) {
+            out.push_back('%');
+            out.push_back(h[(c >> 4) & 0xf]);
+            out.push_back(h[c & 0xf]);
+        } else {
+            out.push_back(char(c));
+        }
+    }
+    return out;
+}
+
+// Note: this differs from smallut stringToBool. Check one day if this is necessary ?
 bool stringToBool(const string& s, bool *value)
 {
     if (s[0] == 'F' ||s[0] == 'f' ||s[0] == 'N' || s[0] == 'n' ||s[0] == '0') {
